#include "Battle.h"
#include "DrawWindow.h"

#include "SDL.h"
#include "SDL_gfxPrimitives.h"
#include "SDL_ttf.h"

#include "InputObj.h"
#include "UserInput.h"

SDL_Surface * textOutput;

Battle :: Battle ()
{
  battleState = 0;
}

void Battle :: init ()
{
  //gets the involved parties, loads stats.
  //update parameters. main calls this at the beginning of a battle.

  endurance = 1000;
  accuracy = 10;
  score = 0;
  battleState = 1;

  if (player1 == NULL)
    player1 = new UserInput ();

  if (dw == NULL)
    dw = new DrawWindow ();
  dw -> init ();
}

void Battle :: addPlayer (UserInput * ui)
{
  player1 = ui;
}

void Battle :: render (SDL_Surface * dest, TTF_Font * font)
{
  char tempStr [80];
  SDL_Color textCol;
  textCol.r = 255;
  textCol.g = 255;
  textCol.b = 255;

  SDL_Rect textPosa;
  textPosa.x = 0;
  textPosa.y = 0;

  SDL_Rect textPos1;
  textPos1.x = 170;
  textPos1.y = 350;
  SDL_Rect textPos2;
  textPos2.x = 170;
  textPos2.y = 369;

  SDL_Rect endFill;
  endFill.x = 170;
  endFill.y = 350;
  endFill.w = 300;
  endFill.h = 15;

  SDL_FillRect(dest, NULL, SDL_MapRGB(dest -> format, 0, 0, 0));

  if (battleState == 1 || battleState == 2)
    {
      dw -> render (dest);

      //draw the "mouse"
      player1 -> render (dest, accuracy);
    }

  sprintf (tempStr, "Hit Escape to Quit");
  textOutput = TTF_RenderText_Solid (font, tempStr, textCol);
  SDL_BlitSurface(textOutput, NULL, dest, &textPosa);

  //endurance
  if (dw -> getDrawingStatus () == 0)
    {
      sprintf (tempStr, "Connect the dots by clicking and dragging the mouse.");      
      textOutput = TTF_RenderText_Solid (font, tempStr, textCol);
      SDL_BlitSurface(textOutput, NULL, dest, &textPos1);
    }

  if (dw -> getDrawingStatus () == 1)
    {
      endFill.w = 300 * getEndurance() / 1000;

      if (getEndurance() > 250)
	SDL_FillRect(dest, &endFill, SDL_MapRGB(dest -> format, 0, 0, 192));
      else
	SDL_FillRect(dest, &endFill, SDL_MapRGB(dest -> format, 192, 0, 0));

      sprintf (tempStr, "Endurance: %d", getEndurance ());
      textOutput = TTF_RenderText_Solid (font, tempStr, textCol);
      SDL_BlitSurface(textOutput, NULL, dest, &textPos1);
      sprintf (tempStr, "Score: %d", getScore ());
      textOutput = TTF_RenderText_Solid (font, tempStr, textCol);
      SDL_BlitSurface(textOutput, NULL, dest, &textPos2);
    }

  if (dw -> getDrawingStatus () == 2)
    {
      sprintf (tempStr, "Game Over! Right click to restart.");
      textOutput = TTF_RenderText_Solid (font, tempStr, textCol);
      SDL_BlitSurface(textOutput, NULL, dest, &textPos1);
      sprintf (tempStr, "Score: %d", getScore ());
      textOutput = TTF_RenderText_Solid (font, tempStr, textCol);
      SDL_BlitSurface(textOutput, NULL, dest, &textPos2);
    }
}

void Battle :: gameLoop ()
{
  //battleState
  //0: Waiting for initialization
  //1: In battle. Draw, attack, etc.
  //2: Battle is over. Can be called for variables.

  switch (battleState){
  case 0:
    break;
  case 1:
    dw -> drawEvaluator (player1 -> getButton1 (), accuracy, 
			 player1 -> getCursorPosX () - dw -> getDimensions().x, 
			 player1 -> getCursorPosY () - dw -> getDimensions().y, 
			 endurance, score);

    if (dw -> getDrawingStatus () == 2)
      battleState = 2;

    break;
  default:
    break;
  }
}

void Battle :: processInput (const SDL_Event & mainEvent)
{
  //give control to whichever player is in action
  player1 -> processInput (mainEvent);
}

int Battle :: getStatus ()
{ return battleState; }

int Battle :: getScore ()
{ return score; }

int Battle :: getEndurance ()
{ return endurance; }

Battle :: ~Battle ()
{}
