#include "DrawWindow.h"
#include "LinePattern.h"

#include "SDL.h"
#include "SDL_gfxPrimitives.h"

DrawWindow :: DrawWindow ()
{
  dimensions.x = 170;
  dimensions.y = 30;
  dimensions.w = 300;
  dimensions.h = 300;
  lp = new LinePattern;
  drawingState = 0;
}

void DrawWindow :: init ()
{
  if (lp != NULL)
    lp -> init (dimensions);

  drawingState = 0;
}

void DrawWindow :: render (SDL_Surface * dest)
{
  SDL_FillRect(dest, &dimensions, SDL_MapRGBA(dest -> format, 0, 0, 128, 255));

  if (lp != NULL)
    lp -> render (dest, dimensions);
}

void DrawWindow :: setLinePattern (LinePattern * llp)
{
  lp = llp;
}

void DrawWindow :: drawEvaluator (bool button1, Uint8 accuracy, Sint16 posX, Sint16 posY, Uint32 & end, Uint32 & score)
{
  //figures out how to handle the mouse's position in relation to the line.

  SDL_Rect endPoint;

  //drawingState keeps track of the status:
  //0: Has not started. Waiting for user to click.
  //1: User has clicked, Has started evaluating.
  //2: User has released/out of endurance. No longer evaluating.

  if (drawingState == 0 && !button1)
    return;

  if (drawingState == 2)
    return;

  if (drawingState == 1 && (!button1 || end <= 0))
    {
      drawingState = 2;
      return;
    }

  if (drawingState == 0 && button1)
    drawingState = 1;

  //OK, evaluate.

  lp -> getLastPoint (endPoint);

  int dx = endPoint.x - posX;
  int dy = endPoint.y - posY;

  int distSq = (dx * dx) + (dy * dy);
  int totalAcc = accuracy + endPoint.w;

  //if mouse is at (or close enough to endPoint...)
  if (distSq <= (totalAcc * totalAcc))
    {
      //tell the line pattern to get the next point.
      lp -> getNewLine (dimensions);

      //increase score.
      score ++;

      //giving an end boost (probably temporary)
      end += 50;
    }

  //reduce endurance here.
  //for now, just reduce it over time.
  end -= 5;
}

SDL_Rect DrawWindow :: getDimensions ()
{
  SDL_Rect * tempSDL = new SDL_Rect (dimensions);
  return (*tempSDL);
}

int DrawWindow :: getDrawingStatus ()
{ return drawingState; }

DrawWindow :: ~DrawWindow ()
{
}
