#include "LinePattern.h"

#include "SDL.h"
#include "SDL_gfxPrimitives.h"
#include <vector>
#include <list>
using namespace std;

//I'm using SDL_Rect as the base structure for the points you're supposed to connect. x, y are the positions.

LinePattern :: LinePattern ()
{}

void LinePattern :: init (SDL_Rect dim)
{
  //restarts the pattern, given pattern rules.
  points.resize (0);

  //other functions assume you have 2 lines
  getNewLine (dim);
}

void LinePattern :: render (SDL_Surface * dest, SDL_Rect dim)
{
  //draw each point on the SDL_Surface dest.

  //as a circle. You can use the w parameter to mark them as 'drawn' and 'not drawn.'

  //draw the current line.
  vector <SDL_Rect> :: iterator iter;

  Uint8 r;
  Uint8 g;
  Uint8 b;
  Uint8 a;

  //make color-choosing functions
  iter = points.begin () + 1;
  while(iter < (points.end())) {
    chooseColor (*iter, dest, r,g,b,a);
    aalineRGBA(dest, (iter - 1) -> x + dim.x, (iter - 1) -> y + dim.y, iter -> x + dim.x, iter -> y + dim.y, r,g,b,a);
    iter ++;
  }

  iter = points.begin ();
  while(iter < points.end()) {
    chooseColor (*iter, dest, r,g,b,a);
    filledCircleRGBA(dest, (*iter).x + dim.x, (*iter).y + dim.y, (*iter).w, r,g,b,255);
    iter ++;
  }
}

int LinePattern :: getLastPoint (SDL_Rect & ptA)
{
  //gets the two current points in the line
  //returns the number of points in the vector.
  //throws an error if there are 0 points in the vector.

  ptA = points.back ();
  return points.size ();
}

void LinePattern :: getNewLine (SDL_Rect dim)
{
  //creates a new point.
  SDL_Rect newPt = generateNextPoint (dim);

  //unmark the previous dot as "current"
  vector <SDL_Rect> :: iterator iter;
  iter = points.begin ();
  while(iter < points.end()) {
    if (iter -> h > 0)
      iter -> h --;
    if (iter -> w > 1)
      iter -> w --;
    iter ++;
  }

  //pop front if too big...
  if (points.size () > 5)
    points.erase (points.begin (), (points.begin () + 2) );

  //mark the new point as "current"
  newPt.w = 5;
  newPt.h = 5;

  //add to queue
  points.push_back (newPt);
}

SDL_Rect LinePattern :: generateNextPoint (SDL_Rect dim)
{
  SDL_Rect newPt;

  //generate new pt according to rules
  newPt.x = rand () % dim.w;
  newPt.y = rand () % dim.h;

  return newPt;
}

Uint32 LinePattern :: chooseColor (SDL_Rect pt, SDL_Surface * dest, Uint8 & r, Uint8 & g, Uint8 & b, Uint8 & a)
{
  //picks the color, based on pt's h parameter.
  Uint32 ans;

  //do math magic here; assumed to be 5 dots long
  r = 192;
  g = 192;
  b = 192;
  a = 255 * pt.h / 5;

  ans = SDL_MapRGBA(dest -> format, r, g, b, a);
  return ans;
}

LinePattern :: ~LinePattern ()
{
}
