#include "SDL.h"
#include "SDL_framerate.h"
#include "SDL_ttf.h"

#include <stdlib.h>
#include <time.h>

#include <vector>
#include <string>
using namespace std;

#include "InputObj.h"
#include "UserInput.h"
#include "Battle.h"

SDL_Surface * screen;
SDL_Surface * mainWindow;
TTF_Font * font;
int screenResX = 640, screenResY = 480;
int mainWindowResX = 640, mainWindowResY = 480;

SDL_Event mainEvent;
FPSmanager fpsKeeper;
int fpsGoal = 30;

Battle battul;
UserInput masterInput;

bool runInDebugMode = false;

Sint8 render ()
{
  //draw stuff on screen

  //lock the framerate.
  SDL_framerateDelay(&fpsKeeper);

  //once it has delayed, draw to the screen and flip it.
  SDL_FillRect(mainWindow, NULL, SDL_MapRGB(mainWindow -> format, 0, 0, 0));

  //call Battle to render this...if in BattleMode.
  battul.render (mainWindow, font);

  //copy main window to screen.
  SDL_BlitSurface(mainWindow, NULL, screen, NULL);

  //finally, flip the screen
  if (SDL_Flip(screen) == -1)
    {
      printf("Cannot flip screens: %s\n", SDL_GetError ());
      return -1;
    }

  return 0;
}

Sint8 gameLoop()
{
  bool done = false;

  while (!done)
    {
      //get events until the queue is empty
      while (SDL_PollEvent(&mainEvent) == 1)
	{
	  //for each event:
	  switch(mainEvent.type){
	  default:
	    break;
	  }

	  //only if in battle mode
	  battul.processInput (mainEvent);

	  masterInput.processInput (mainEvent);
	}
      //done with all events

      //execute pending input decisions here
      //tell battle to do stuff here
      battul.gameLoop ();

      //this is used for main.
      //check to see if it wants to quit
      if(masterInput.getEscKey ())
	done = true;
      
      //make sure this is only when in Battul Mode.
      if (masterInput.getButton2 () && battul.getStatus () == 2)
	{ battul.init (); }

      //perform results of logic here

      //draw the screen
      if (render () == -1)
	return -1;
    }
  return 0;
}

Sint8 loadRound()
{
  //set up the rules.
  return 0;
}

Sint8 initGameVariables()
{
  //set the framerate
  SDL_initFramerate(&fpsKeeper);
  SDL_setFramerate(&fpsKeeper, fpsGoal);

  //init keyboard
  SDL_EnableKeyRepeat(1, 30);

  //set the randomizer seed.
  srand( time(NULL) );
  return 0;
}

Sint8 initGame()
{
  if (initGameVariables() == -1)
    return -1;

  if (loadRound () == -1)
    return -1;

  return 0;
}

Sint8 initGfx(bool debugMode)
{
  //set up screen
  if (debugMode)
    screen = SDL_SetVideoMode(640, 480, 32, SDL_HWSURFACE|SDL_DOUBLEBUF);
  else
    screen = SDL_SetVideoMode(640, 480, 32, SDL_HWSURFACE|SDL_DOUBLEBUF|SDL_FULLSCREEN);

  if (screen == NULL)
    {
      printf ("Error initializing video, Attempting software surface.");
      if (debugMode)
	screen = SDL_SetVideoMode(screenResX, screenResY, 32, SDL_SWSURFACE|SDL_DOUBLEBUF);
      else
	screen = SDL_SetVideoMode(screenResX, screenResY, 32, SDL_SWSURFACE|SDL_DOUBLEBUF|SDL_FULLSCREEN);
    }

  if (screen == NULL)
    {
      printf ("Error initializing video: %s", SDL_GetError ());
      return -1;
    }

  //setup mainWindow
  mainWindow = SDL_CreateRGBSurface (SDL_HWSURFACE, screenResX, screenResY, 32, screen -> format -> Rmask, screen -> format -> Gmask, screen -> format -> Bmask, screen -> format -> Amask);

  battul.init ();

  //disbale mouse
  SDL_ShowCursor(SDL_DISABLE);
  return 0;
}

Sint8 initSDL(bool debugMode)
{
  if (SDL_Init(SDL_INIT_VIDEO) == Sint8(-1))
    {
      printf ("Unable to init SDL: %s\n", SDL_GetError());
      return -1;
    }

  if(TTF_Init()==-1) 
    {
      printf("Unable to init SDL_TTF: %s\n", TTF_GetError());
      return -1;
    }

  font=TTF_OpenFont("text/Times_New_Roman.ttf", 12);
  
  if(font == NULL) 
    {
      printf("TTF_OpenFont: %s\n", TTF_GetError());
      return -1;
    }
  return 0;
}

Sint8 initEngine()
{
  if (initSDL(runInDebugMode) == -1)
    return -1;

  if (initGfx (runInDebugMode) == -1)
    return -1;

  return 0;
}

int main (int argc, char *argv[])
{
  if (initEngine() == -1)
    return -1;

  if (initGame () == -1)
    return -1;

  if (gameLoop () == -1)
    return -1;

  SDL_ShowCursor(SDL_ENABLE);
  SDL_FreeSurface(mainWindow);
  SDL_Quit ();
  TTF_Quit ();
  return 0;
};
